###############################################################################
# pytest_opensubset.py: Open spatial subset based on extent of vector dataset
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/23 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input file",dest="input",required=True,type=str)
parser.add_argument("-extent","--extent",help="Path of the extent file",dest="extent",required=True,type=str)
args = parser.parse_args()

jim0=jl.createJim(filename=args.input,extent=args.extent)
v0=jl.createVector()
v0.open({'filename':args.extent})
print("v0.getBoundingBox():",v0.getBoundingBox())
if jim0.getUlx()<v0.getUlx()-jim0.getDeltaX() or jim0.getUlx()>v0.getUlx()+jim0.getDeltaX():
    print("1Failed: get spatial extent ulx")
elif jim0.getUly()>v0.getUly()+jim0.getDeltaY() or jim0.getUly()<v0.getUly()-jim0.getDeltaY():
    print("2Failed: get spatial extent uly")
if jim0.getLrx()<v0.getLrx()-jim0.getDeltaX() or jim0.getLrx()>v0.getLrx()+jim0.getDeltaX():
    print("3Failed: get spatial extent lrx")
elif jim0.getLry()>v0.getLry()+jim0.getDeltaY() or jim0.getLry()<v0.getLry()-jim0.getDeltaY():
    print("4Failed: get spatial extent lry")
else:
    print("Success: createJim with extent")
jim0.close()
v0.close()




###############################################################################
# pytest_extractogr.py: extractogr
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/24 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import math
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the raster dataset",dest="input",required=False,type=str)
parser.add_argument("-ulx","--ulx",help="left margin of bounding box",dest="ulx",required=False,type=float,default=16.1)
parser.add_argument("-lrx","--lrx",help="lrx margin of bounding box",dest="lrx",required=False,type=float,default=16.6)
parser.add_argument("-uly","--uly",help="uly margin of bounding box",dest="uly",required=False,type=float,default=48.6)
parser.add_argument("-lry","--lry",help="lry margin of bounding box",dest="lry",required=False,type=float,default=47.2)
parser.add_argument("-dx","--dx",help="Resolution in x",dest="dx",required=False,type=float)
parser.add_argument("-dy","--dy",help="Resolution in y",dest="dy",required=False,type=float)
parser.add_argument("-t_srs","--t_srs",help="Target spatial reference system of bounding box",dest="t_srs",required=False,type=str,default='4326')
parser.add_argument("-output","--output",help="Path of the output vector dataset",dest="output",required=True,type=str)
parser.add_argument("-random","--random",help="Number of random pixels to select",dest="random",required=False,type=int,default=10)
args = parser.parse_args()

# try:
if True:
    rules=['median']
    if args.output:
        output=args.output
        oformat='SQLite'
    else:
        output='mem01'
        oformat='Memory'

    openDict={'t_srs':'epsg:'+args.t_srs}
    openDict.update({'ulx':args.ulx,'uly':args.uly,'lrx':args.lrx,'lry':args.lry})
    if(args.dx):
        openDict.update({'dx':args.dx})
    if(args.dy):
        openDict.update({'dy':args.dy})
    if args.input:
        refpath=args.input
    else:
        refpath='/eos/jeodpp/data/base/Landcover/EUROPE/CorineLandCover/CLC2012/VER18-5/Data/GeoTIFF/250m/g250_clc12_V18_5.tif'
        openDict.update({'s_srs':'epsg:3035'})

    # openDict.update({'filename':refpath})
    jim_ref=jl.createJim(refpath,**openDict)
    # jim_ref=jim_ref.warp({'t_srs':'epsg:'+args.t_srs})

    classDict={}
    classDict['urban']=2
    classDict['agriculture']=12
    classDict['forest']=25
    classDict['water']=41
    classDict['rest']=50
    sorted(classDict.values())

    print(classDict)

    classFrom=list(range(0,50))
    classTo=[50]*50
    print("classFrom: {}".format(classFrom))
    print("classTo: {}".format(classTo))
    for i in range(0,50):
        if i>=1 and i<10:
            classTo[i]=classDict['urban']
        elif i>=11 and i<22:
            classTo[i]=classDict['agriculture']
        elif i>=23 and i<25:
            classTo[i]=classDict['forest']
        elif i>=40 and i<45:
            classTo[i]=classDict['water']
        else:
            classTo[i]=classDict['rest']

    jim_ref=jim_ref.reclass({'class':classFrom,'reclass':classTo})
    jim_ref.write({'filename':'/vsimem/reference.tif'})

    labels=classDict.values()
    print(labels)
    for classname in classDict:
        print("class: ",classname)
        label=classDict[classname]
        srcnodata=list(classDict.values())
        srcnodata.remove(label)
        srcnodata.append(255)
        print(srcnodata)
        print("extract")
        v=jim_ref.extractSample({'ln':classname,'random':args.random,'rule':rules,'output':output,'oformat':oformat,'bandname':['label'],'mask':'/vsimem/reference.tif','msknodata':srcnodata,'buffer':1,'co':'OVERWRITE=YES'})
        print("write")
        v.write()
        print("close")
    v.close()
    jim_ref.close()
    print("Success: extractogr")
try:
    print("debug0")
except:
    print("Failed: extractogr")



###############################################################################
# pytest_write.py: open raster dataset and write to file
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/23 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input file",dest="input",required=True,type=str)
parser.add_argument("-output","--output",help="Path of the output file",dest="output",required=True,type=str)
args = parser.parse_args()

try:
    jim0=jl.createJim(args.input)
    jim0.write({'filename':args.output, 'oformat': 'GTiff', 'co':['COMPRESS=LZW','TILED=YES']}).close()
    if os.path.isfile(args.output):
        print("Success: write to file")
    else:
        throw()
except:
    print("Failed: write()")




###############################################################################
# pytest_statprofile.py: statprofile
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/12/13 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import math
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the raster dataset",dest="input",required=True,type=str)
parser.add_argument("-function","--function",help="Statistical functions to perform",dest="function",required=True,type=str,nargs='+')
parser.add_argument("-perc","--perc",help="Percentiles to calculate",dest="percentile",required=False,type=str,nargs='*')
parser.add_argument("-output","--output",help="Path of the output vector dataset",dest="output",required=False,type=str)
args = parser.parse_args()

try:
    print('calculating functions:',args.function)
    jim0=jl.createJim(args.input)
    if args.percentile:
        jim1=jim0.statProfile({'function':args.function,'perc':args.percentile})
    else:
        jim1=jim0.statProfile({'function':args.function})
    if jim1.nrOfBand() < len(args.function):
        throw()
    if args.output:
        jim1.write({'filename':args.output})
    jim1.close()
    jim0.close()
    print("Success: statprofile")
except:
    print("Failed: statprofile")



# first 20161111 by Pierre.Soille@jrc.ec.europa.eu

def fun2method(inputfile, outputfile_basename, json_path):
    """converts MIALib C function declarations into JIPLib C++ methods (outputfile_basename.cc file) and C++ method declarations (outputfile_basename.h file).  Currently only convert desctuctive functions, i.e., ERROR_TYPE functions, with IMAGE * as first argument (IMAGE ** not yet taken into account).

    :param inputfile: string for input file containing extern declarations
    :param outputfile_basename: string for output file basename (i.e. without extension)
    :returns: True on success, False otherwise
    :rtype:

    """

    import re
    import json
    import sys 
    import os
    #json_path = sys.argv[3] # or adjust based on your arg parsing
    with open(json_path, 'r') as f:
        old2newDic = json.load(f)

    #old2newDic = json.load(open("old2NewNames.json"))

    ifp=open(inputfile, 'r')

    f = open(outputfile_basename+'.cc', 'w')
    fh = open(outputfile_basename+'.h', 'w')


    lines=ifp.readlines()

    for line in lines:
        print(line)

        name=re.match(r'extern IMAGE \*(.*)\((.*)\);',line)
        args=re.split(',', name.group(2))
        re.sub('\**', '', re.sub('.* ', '', args[0]))

        arglist = []

        for i in args:
            atype=re.match(' *(.* \**)', i).group(1)
            aname=re.sub('\**', '', re.sub('.* ', '', i))
            arglist.append([atype, aname])


        a= { "name" : name.group(1),
             "arguments" : arglist }

        print(a)

        imDeclare=[]
        imRasterArray=[]

        GTDeclare=[]
        GTVars=[]

        GDTypes = ['GDT_Byte', 'GDT_Int16', 'GDT_UInt16', 'GDT_Int32', 'GDT_UInt32', 'GDT_Float32', 'GDT_Float64']
        MIATypes = ['uc_', 's_', 'us_', 'i32_', 'u32_', 'f_', 'd_']
        CTypes = ['unsigned char', 'short int', 'unsigned short int', 'int', 'unsigned int', 'float', 'double']

        methodDeclaration='std::shared_ptr<Jim> Jim::'+old2newDic.get(a.get("name"))+'('
        print(methodDeclaration)

        cSeparator=', '
        separator = '' # default value in case there are no arguments besides the input image

        if a.get("arguments")[0][0] != 'IMAGE *':
            print('WARNING: ',line,' does not have IMAGE * as first argument')
            continue
        else:
            imDeclare.append(a.get("arguments")[0][0]+' '+a.get("arguments")[0][1]+'=this->getMIA(iband);')
            cCall=a.get("arguments")[0][1]

        imCount=1
        for idx, arg in enumerate(a.get("arguments")[1:len(a.get("arguments"))]):
            print(methodDeclaration)
            print(idx)
            print(idx)
            print(separator)
            print(arg)
            print(arg[0])

            if (arg[0]=='IMAGE *'):
                methodDeclaration+=separator+'Jim& imRaster_'+arg[1]
                imRasterArray.append('imRaster_'+arg[1])
                imDeclare.append(arg[0]+' '+arg[1]+'=imRaster_'+arg[1]+'.getMIA(iband);')
            elif (arg[0]=='G_TYPE '): # G_TYPE as double in python
                GTDeclare.append(arg[0]+' '+arg[1]+';')
                GTVars.append(arg[1])
                methodDeclaration+=separator+'double '+'d_'+arg[1]
            else:
                methodDeclaration+=separator+arg[0]+' '+arg[1]

            cCall+=cSeparator+arg[1]
            separator=', '

        fh.write(re.sub(r'Jim::','',methodDeclaration+separator+'int iband=0);\n'))

        methodDeclaration+=separator+'int iband)'

        f.write(methodDeclaration+'{')
        f.write('\n\ttry{')
        f.write('\n\t\tif(nrOfBand()<=iband){')
        f.write('\n\t\t\tstd::string errorString=\"Error: band number exceeds number of bands in input image\";')
        f.write('\n\t\t\tthrow(errorString);')
        f.write('\n\t\t}')
        for i in imRasterArray:
            f.write('\n\t\tif('+i+'.nrOfBand()<=iband){')
            f.write('\n\t\t\tstd::string errorString=\"Error: band number exceeds number of bands in input image\";')
            f.write('\n\t\t\tthrow(errorString);')
            f.write('\n\t\t}')
        #f.write('\n\t\t'+a.get("arguments")[0][0]+' '+a.get("arguments")[0][1]+' = 0;') # assigned later depending on destructive or not
        f.write('\n\t\t'+'IMAGE * imout = 0;')
        llen=len(imDeclare)
        for i in range(0,llen):
            f.write('\n\t\t'+imDeclare[i])
        for i in GTDeclare:
            f.write('\n\t\t'+i)

        if len(GTDeclare)>0:
            f.write('\n\t\tswitch(getDataType()){')
            for idx, GDType in enumerate(GDTypes):
                f.write('\n\t\tcase('+GDType+'):')
                for var_idx, GTVar in enumerate(GTVars):
                    f.write('\n\t\t\t'+GTVar+'.'+MIATypes[idx]+'val=static_cast<'+CTypes[idx]+'>(d_'+GTVars[var_idx]+');')
                    if (GDType=='GDT_Byte'):
                        f.write('\n\t\t\t'+GTVar+'.'+'generic_'+'val=static_cast<'+CTypes[idx]+'>(d_'+GTVars[var_idx]+');')
                f.write('\n\t\t\tbreak;')
            f.write('''
    \t\tdefault:
    \t\t\tstd::string errorString="Error: data type not supported";
    \t\t\tthrow(errorString);
    \t\t\tbreak;
            \t\t}''')


        f.write('\n\t\timout =::'+a.get("name")+'('+cCall+');')

        f.write('\n\t\t\tthis->setMIA(iband);')
        for i in imRasterArray:
            f.write('\n\t\t\t\tif(this!=&'+i+')')
            f.write('\n\t\t\t\t\t'+i+'.setMIA(iband);')

        f.write('\n\t\t\tif (imout){')
        f.write('\n\t\t\t\tstd::shared_ptr<Jim> imgWriter=std::make_shared<Jim>(imout);')
        f.write('\n\t\t\t\timgWriter->copyGeoTransform(*this);')
        f.write('\n\t\t\t\timgWriter->setProjection(getProjectionRef());')
        f.write('\n\t\t\t\treturn(imgWriter);')
        f.write('\n\t\t\t}')


        f.write('\n\t\t\telse{')
        f.write('\n\t\t\t\tstd::string errorString="Error: '+a.get("name")+'() function in MIA failed, returning NULL pointer";')
        f.write('\n\t\t\t\tthrow(errorString);')
        f.write('\n\t\t\t}')

        f.write('\n\t\t}')


        f.write('''
        catch(std::string errorString){
        \tstd::cerr << errorString << std::endl;
        throw;
        }
        catch(...){
        throw;
        }
 }\n''')

    ifp.close()

    f.close()
    fh.close()

    return(True)



####################################################################

import sys, getopt

def main(argv):
   inputfile="miallib_imagetype"
   outputfile="fun2method_imagetype"
   json_path = "old2NewNames.json"
   try:
       opts, args = getopt.getopt(argv,"hi:o:j:",["ifile=","ofile=", "json="])
   except getopt.GetoptError:
      print('fun2method.py -i <inputfile> -o <outputfilebasename> -j <json_path>')
      sys.exit(2)
   for opt, arg in opts:
      if opt == '-h':
         print('fun2method.py -i <inputfile> -o <outputfilebasename> -j <json_path>')
         sys.exit()
      elif opt in ("-i", "--ifile"):
         inputfile = arg
      elif opt in ("-o", "--ofile"):
         outputfile = arg
      elif opt in ("-j", "--json"):
         json_path = arg
   print('Input file is "', inputfile)
   print('Output file is "', outputfile)
   print('JSON file is "', json_path)

   fun2method(inputfile, outputfile, json_path)

if __name__ == "__main__":
   main(sys.argv[1:])


# cat /usr/local/include/miallib/miallib_*.h | grep '^extern IMAGE \*[^\*]'  > miallib_image_type
# python fun2method_imagetype.py  -i miallib_image_type -o fun2method_imagetype
# to automatically insert content of fun2method in jim.h within placeholder //start insert from fun2method -> //end insert from fun2method
# sed -i -ne '/\/\/start insert from fun2method_imagetype/ {p; r fun2method_imagetype.h' -e ':a; n; /\/\/end insert from fun2method_imagetype/ {p; b}; ba}; p' jim.h



###############################################################################
# pytest_createsimple.py: Create a simple image
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/23 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-nrow","--nrow",help="Number of rows",dest="nrow",required=False,type=int,default=1024)
parser.add_argument("-ncol","--ncol",help="Number of cols",dest="ncol",required=False,type=int,default=1024)
args = parser.parse_args()

dict={'nrow':args.nrow,'ncol':args.ncol}
jim0=jl.createJim(**dict)
if jim0.nrOfCol()!=args.ncol:
    print("Failed: number of cols")
if jim0.nrOfRow()!=args.nrow:
    print("Failed: number of rows")
else:
    print("Success: create simple image with nrow and ncol")
jim0.close()




###############################################################################
# pytest_classifyOgr.py: classify vector dataset
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2018/03/07 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import math
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input raster dataset",dest="input",required=True,type=str)
parser.add_argument("-vector","--vector",help="Path of the sample vector dataset with labels",dest="vector",required=True,type=str)
parser.add_argument("-model","--model",help="Path of the model output filename used for training",dest="model",required=True,type=str)
parser.add_argument("-output","--output",help="Path of the classification output raster dataset",dest="output",required=True,type=str)
parser.add_argument("-classifier","--classifier",help="classifier (svm, ann)",dest="classifier",required=False,type=str,default="svm")
args = parser.parse_args()

try:
    print("createJim")
    jim=jl.createJim(args.input)
    print("createVector")
    sample=jl.createVector();
    print("open vector",args.vector)
    sample.open(args.vector)
    print("extractOgr")
    training=jim.extractOgr(sample,{'output':'training','oformat':'Memory','copy':'label'})
    test=jim.extractOgr(sample,{'output':'test','oformat':'Memory','copy':'None'})
    if args.classifier == 'svm':
        #SVM classification
        print("training")
        training.train({'method':args.classifier,'label':'label','model':args.model})
        print("classification")
        vclass=test.classify({'output':args.output,'co':'OVERWRITE=YES','model':args.model,'method':args.classifier})
        print("write")
        vclass.write()
        vclass.close()
    else:
        #ANN classification
        print("training")
        training.train({'method':args.classifier,'label':'label','model':args.model})
        print("classification")
        vclass=test.classify({'output':args.output,'co':'OVERWRITE=YES','model':args.model,'method':args.classifier})
        print("write")
        vclass.write()
        vclass.close()
        sample.close()
    test.close()
    training.close()
    jim.close()
    print("Success: classify")
except:
    print("Failed: classify")




###############################################################################
# pytest_setthreshold.py: setThreshold to create mask
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
#
# This file is part of jiplib
###############################################################################

# History
# 2017/10/24 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input file",dest="input",required=True,type=str)
parser.add_argument("-min","--min",help="min value for threshold",dest="min",required=True,type=int)
parser.add_argument("-max","--max",help="max value for threshold",dest="max",required=True,type=int)
args = parser.parse_args()

jim0=jl.createJim(args.input)
if(args.min==0 and args.max==1):
    try:
        jim0=jim0.pushNoDataValue(1)
        # Create binary mask where all values not within [0,1[ are set to 1 (i.e., all pixel values > 0 are set to 1)
        jim0=jim0.setThreshold({'min':args.min,'max':args.max})
        if jim0.getNvalid()!=87957:
            print("Failed: nvalid")
            throw()
        if jim0.getNinvalid()!=174187:
            print("Failed: ninvalid")
            throw()
        print("Success: masking")
    except:
        print("Failed: masking")
elif(args.min==10 and args.max==50):
    try:
        jim0=jim0.pushNoDataValue(0)
        jim1=jim0.setThreshold({'min':10,'max':50})
        theStats=jim1.getStats({'function':['min','max'],'nodata':0})
        print(theStats)
        if theStats['min'][0]!=10:
            print("Failed: min",theStats['min'])
            throw()
        if theStats['max'][0]!=50:
            print("Failed: max",theStats['max'])
            throw()
        jim1=jim0.setThreshold({'min':10,'max':50,'value':100}).clearNoData()
        theStats=jim1.getStats({'function':['min','max']})
        if theStats['min'][0]!=0:
            print("Failed: min",theStats['min'])
            throw()
        if theStats['max'][0]!=100:
            print("Failed: max",theStats['max'])
            throw()
        print("Success: masking")
    except:
        print("Failed: masking")
else:
    try:
        jim0=jim0.pushNoDataValue(0)
        jim1=jim0.setThreshold({'min':args.min,'max':args.max})
        theStats=jim1.getStats({'function':['min','max'],'nodata':0})
        print(theStats)
        print("Success: masking")
    except:
        print("Failed: masking")
    jim1.close()
jim0.close()



###############################################################################
# pytest_composite.py: composite min/max
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/24 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the raster dataset",dest="input",required=True,type=str)
args = parser.parse_args()

try:
    jim0=jl.createJim(args.input)
    stats0=jim0.getStats({'function':['min','max'], 'band':0})
    stats5=jim0.getStats({'function':['min','max'], 'band':5})
    stats10=jim0.getStats({'function':['min','max'], 'band':10})
    jl0=jl.JimList([jim0.cropBand({'band':0}),jim0.cropBand({'band':5}),jim0.cropBand({'band':10})])
    print("stack bands")
    # jimstack=jl0.stackBand({'verbose':2})
    jimstack=jl0.stackBand()
    print("creating min composite")
    jim1=jl0.composite({'crule':'minband'}).clearNoData()
    print("get min statistics")
    statmin=jim1.getStats({'function':['min']})
    if statmin['min']>min(stats0['min'],stats5['min'],stats10['min']):
        print("Failed: min",statmin['min'],stats0['min'],stats5['min'],stats10['min'])
        throw()
    print("creating statistics profile")
    jimsp=jimstack.statProfile({'function':'min'})
    if not jim1.isEqual(jimsp):
        print("Failed: statProfile min")
        throw()
    print("creating max composite")
    jim1=jl0.composite({'crule':'maxband'}).clearNoData()
    jl0.close()
    print("get max statistics")
    statmax=jim1.getStats({'function':['max']})
    if statmax['max']<max(stats0['min'],stats5['min'],stats10['min']):
        print("Failed: max",statsc['max'],stats0['max'],stats5['max'],stats10['max'])
        throw()
    print("creating statistics profile")
    jimsp=jimstack.statProfile({'function':'max'})
    if not jim1.isEqual(jimsp):
        print("Failed: statProfile max")
        throw()
    jl0.close()
    jim1.close()
    jimsp.close()
    print("Success: composite min/max")
except:
    print("Failed: composite min/max")



###############################################################################
# pytest_randuniform.py: Create a georeferenced image with random pixel values using uniform distribution
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/23 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log

import argparse
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-nrow","--nrow",help="Number of rows",dest="nrow",required=False,type=int,default=1024)
parser.add_argument("-ncol","--ncol",help="Number of cols",dest="ncol",required=False,type=int,default=1024)
parser.add_argument("-min","--min",help="min value for pixel values",dest="min",required=False,type=float,default=1000)
parser.add_argument("-max","--max",help="max value for pixel values",dest="max",required=False,type=float,default=2000)
args = parser.parse_args()

ULX=600000.0
ULY=4000020.0
LRX=709800.0
LRY=3890220.0
projection='epsg:32612'
dict={'ulx':ULX,'uly':ULY,'lrx':LRX,'lry':LRY,'a_srs':projection}
dict.update({'nrow':args.nrow,'ncol':args.ncol})
dict.update({'uniform':[args.min,args.max+1]})
dict.update({'otype':'GDT_UInt16'})
dict.update({'seed':10915})
jim0=jl.createJim(**dict)
theStats=jim0.getStats({'function':['min','max']})
if theStats['min']!=args.min:
    print("Failed: min")
if theStats['max']!=args.max:
    print("Failed: max")
else:
    print("Success: create georeferenced image with random pixel values using uniform distribution")
jim0.close()



###############################################################################
# pytest_init.py: Create a georeferenced image with initialized pixel values
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/23 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log

import argparse
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-nrow","--nrow",help="Number of rows",dest="nrow",required=False,type=int,default=1024)
parser.add_argument("-ncol","--ncol",help="Number of cols",dest="ncol",required=False,type=int,default=1024)
parser.add_argument("-value","--value",help="Initial value for image pixels",dest="value",required=False,type=float,default=1000)
args = parser.parse_args()

ULX=600000.0
ULY=4000020.0
LRX=709800.0
LRY=3890220.0
projection='epsg:32612'
dict={'ulx':ULX,'uly':ULY,'lrx':LRX,'lry':LRY,'a_srs':projection}
dict.update({'otype':'GDT_Float32'})
nrow=1098
ncol=1098
# dict.update({'nrow':args.nrow,'ncol':args.ncol,'mean':args.value})
dict.update({'nrow':args.nrow,'ncol':args.ncol})
jim0=jl.createJim(**dict)
print("gds before pointOpBlank:{}".format(jim0.getDataset()))
jim0=jim0.pointOpBlank(args.value)
print("gds after pointOpBlank:{}".format(jim0.getDataset()))
theStats=jim0.getStats({'function':['min','max']})
print("gds:{}".format(jim0.getDataset()))
print("statistics:{}".format(theStats))
if theStats['max']!=args.value:
    print("Failed: max")
if theStats['min']!=args.value:
    print("Failed: min")
else:
    print("Success: create georeferenced image with initialized pixel values")
print("gds:{}".format(jim0.getDataset()))
jim0.close()



"""
Install the jiplib package.
Author(s): Pieter.Kempeneers@ec.europa.eu,
           Ondrej Pesek,
           Pierre.Soille@ec.europa.eu
Copyright (C) 2018-2026 European Union (Joint Research Centre)

This file is part of jiplib.

jiplib is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

jiplib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
"""
from __future__ import absolute_import
from .jiplib import *


__version__ = '1.2.0'



###############################################################################
# pytest_getmask.py: getMask
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/24 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input file",dest="input",required=True,type=str)
args = parser.parse_args()

try:
    jim0=jl.createJim(args.input)
    theStats=jim0.getStats({'function':'max'})
    # The method getMask sets values within [min,max] to value defined by 'data', else to 'nodata'
    # The min and max values can be defined as a list
    jim1=jim0.getMask({'min':1,'max':theStats['max'][0],'nodata':0,'data':1}).pushNoDataValue(1)
    if jim1.getNvalid()!=87957:
        print("Failed: nvalid",jim1.getNvalid())
        throw()
    if jim1.getNinvalid()!=174187:
        print("Failed: ninvalid",jim1.getNinvalid())
        throw()
    # The min and max values can be defined as a list (min and max values should correspond element wise)
    # We create a mask with pixel value=1 for pixels within [1,1000] and [2000,maxValue]
    # Pixels not within those ranges get a no data value 0
    jim1=jim0.getMask({'min':[1,30],'max':[10,theStats['max'][0]],'nodata':0,'data':1}).setNoDataValue(0)
    if jim1.getNvalid()!=112924:
        print("Failed: nvalid",jim1.getNvalid())
        throw()
    if jim1.getNinvalid()!=149220:
        print("Failed: ninvalid",jim1.getNinvalid())
        throw()
    jim0.close()
    jim1.close()
    print("Success: getMask")
except:
    print("Failed: getMask")
    jim0.close()



"""
Install the jiplib package.
Author(s): Pieter.Kempeneers@ec.europa.eu,
Copyright (C) 2018-2026 European Union (Joint Research Centre)

This file is part of jiplib.

jiplib is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

jiplib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
"""

try:
    from setuptools import setup, Distribution
except ImportError:
    from distutils.core import setup

from setuptools import find_packages

setup(
    name='jiplib',
    version='1.2.0',
    author='Pieter Kempeneers',
    author_email='pieter.kempeneers@ec.europa.eu',
    url='https://jeodpp.jrc.ec.europa.eu/apps/gitlab/JIPlib/jiplib',
    license='GPLv3',
    packages=find_packages(),
    package_data={'jiplib': ['_jiplib.so', 'libjiplib.so.1',
                             'libmiallib.so.1']},
    include_package_data=True,
    # long_description=open('jiplib/README.md').read(),
    # long_description_content_type='text/markdown',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Image Processing',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.8',
    ],
    python_requires='>=3.8',
)



###############################################################################
# pytest_classify_s2.py: classify
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
#
# This file is part of jiplib
###############################################################################

# History
# 2018/01/22 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import math
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input raster dataset",dest="input",required=True,type=str)
parser.add_argument("-reference","--reference",help="Path of the reference raster dataset",dest="reference",required=True,type=str)
parser.add_argument("-model","--model",help="Path of the model output filename used for training",dest="model",required=False,type=str)
parser.add_argument("-training","--training",help="Path of the training vector dataset with raster information",dest="training",required=False,type=str)
parser.add_argument("-output","--output",help="Path of the classification output raster dataset",dest="output",required=False,type=str)
parser.add_argument("-sampleSize","--sampleSize",help="Sample size used for training svm or ann",dest="sampleSize",required=False,type=int,default=100)
parser.add_argument("-classifier","--classifier",help="classifier (sml, svm, ann)",dest="classifier",required=True,type=str,default="sml")
args = parser.parse_args()

# try:
if True:
    jim=jl.createJim(args.input)
    #preparation of reference
    classDict={}
    classDict['urban']=2
    classDict['agriculture']=12
    classDict['forest']=25
    classDict['water']=41
    classDict['rest']=50
    classFrom=list(range(0,50))
    classTo=[50]*50
    for i in range(0,50):
        if i>=1 and i<10:
            classTo[i]=classDict['urban']
        elif i>=11 and i<22:
            classTo[i]=classDict['agriculture']
        elif i>=23 and i<25:
            classTo[i]=classDict['forest']
        elif i>=40 and i<45:
            classTo[i]=classDict['water']
        else:
            classTo[i]=classDict['rest']

    print("jim.getProjection() {}".format(jim.getProjection()))
    jim_ref=jl.createJim(filename=args.reference,dx=jim.getDeltaX(),dy=jim.getDeltaY(),ulx=jim.getUlx(),uly=jim.getUly(),lrx=jim.getLrx(),lry=jim.getLry(),t_srs=jim.getProjection())
    jim_ref=jim_ref.reclass({'class':classFrom,'reclass':classTo})

    if args.classifier == "sml":
        print("SML classifier")
        reflist=jl.JimList([jim_ref])
        jim.d_band2plane();
        # jim.train(reflist,{'method':'sml','model':args.model,'class':sorted(classDict.values()),'verbose':1})
        # sml=jim.classify({'method':'sml','model':args.model,'verbose':1})
        sml=jim.classifySML(reflist,{'class':sorted(classDict.values()),'verbose':1})
        sml.write({'filename':'/tmp/sml_classes.tif'})
        # #preparation of output
        sml_class=sml.statProfile({'function':'maxindex'}).reclass({'class':list(range(0,sml.nrOfBand())),'reclass':sorted(classDict.values())})
        if args.output:
            sml_class.write({'filename':args.output})
        sml_class.close()
    else:
        srcnodata=[0]
        dict={'srcnodata':srcnodata}
        if args.training:
            trainingfn=args.training
            dict.update({'oformat':'ESRI Shapefile'})
        else:
            dict.update({'oformat':'Memory'})
            trainingfn='training.shp'
        dict.update({'output':trainingfn})
        dict.update({'class':sorted(classDict.values())})
        sampleSize=-args.sampleSize #use negative values for absolute and positive values for percentage values
        dict.update({'threshold':sampleSize})
        dict.update({'bandname':['B02','B03','B04','B08']})
        # dict.update({'band':[0,1,2,3]})
        # dict.update({'verbose':2})
        sample=jim.extractImg(jim_ref,dict)
        if args.classifier == "svm":
            #training
            #explicitly define all band names to use for training
            sample.train({'method':args.classifier,'label':'label','bandname':['B02','B03','B04','B08'],'model':args.model})
            #define selection of band names to use for training
            #sample.train({'method':'svm','label':'label','bandname':['B03','B08'],'model':args.model})
            sample.close()
            #classification
            svm_class=jim.classify({'method':'svm','model':args.model})
            #explicitly define all band indexes to use for classification
            #svm_class=jim.classify({'method':'svm','model':args.model,'band':[0,1,2,3]})
            #define selection of band indexes to use for classification
            #svm_class=jim.classify({'method':'svm','model':args.model,'band':[1,3]})
            if args.output:
                svm_class.write({'filename':args.output})
            svm_class.close()
        elif args.classifier == "ann":
            #training
            sample.train({'method':args.classifier,'label':'label','model':args.model})
            #explicitly define all band names to use for training
            #sample.train({'method':'ann','label':'label','bandname':['B02','B03','B04','B08'],'model':args.model})
            #define selection of band names to use for training
            #sample.train({'method':'ann','label':'label','bandname':['B02','B04'],'model':args.model})
            sample.close()
            #classification
            ann_class=jim.classify({'method':'ann','model':args.model})
            #explicitly define all band indexes to use for classification
            #svm_class=jim.classify({'method':'ann','model':args.model,'band':[0,1,2,3]})
            #define selection of band indexes to use for classification
            #ann_class=jim.classify({'method':'ann','model':args.model,'band':[0,2]})
            if args.output:
                ann_class.write({'filename':args.output})
            ann_class.close()
        else:
            print("Error: classifier",args.classifier,"not implemented")
            throw()

    jim.close()
    print("Success: classify")
try:
    print("ok")
except:
    print("Failed: classify")
    jim.close()



###############################################################################
# pytest_randgaussian.py: Create a georeferenced image with random pixel values
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/23 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log

import argparse
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-nrow","--nrow",help="Number of rows",dest="nrow",required=False,type=int,default=1024)
parser.add_argument("-ncol","--ncol",help="Number of cols",dest="ncol",required=False,type=int,default=1024)
parser.add_argument("-mean","--mean",help="mean value for pixel values",dest="mean",required=False,type=float,default=1000)
parser.add_argument("-stdev","--stdev",help="stdev value for pixel values",dest="stdev",required=False,type=float,default=100)
args = parser.parse_args()

ULX=600000.0
ULY=4000020.0
LRX=709800.0
LRY=3890220.0
projection='epsg:32612'
dict={'ulx':ULX,'uly':ULY,'lrx':LRX,'lry':LRY,'a_srs':projection}
dict.update({'otype':'GDT_Float32'})
dict.update({'nrow':args.nrow,'ncol':args.ncol})
dict.update({'mean':args.mean,'stdev':args.stdev})
jim0=jl.createJim(**dict)
theStats=jim0.getStats({'function':['mean','stdev']})
print(theStats)
if theStats['mean']<args.mean-1 or theStats['mean']>args.mean+1:
    print("Failed: mean")
if theStats['stdev']<args.stdev-1 or theStats['stdev']>args.stdev+1:
    print("Failed: stdev")
else:
    print("Success: create georeferenced image with random pixel values")
jim0.close()



###############################################################################
# pytest_crop.py: crop
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2018/03/01 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import math
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the raster dataset",dest="input",required=True,type=str)
parser.add_argument("-extent","--extent",help="Path of the vector dataset",dest="extent",required=False,type=str)
parser.add_argument("-output","--output",help="Path of the output vector dataset",dest="output",required=False,type=str)
parser.add_argument("-nodata","--nodata",help="nodata value to put in cut region",dest="nodata",required=False,default=0,type=int)
parser.add_argument("-cut_to_cutline","--cut_to_cutline",help="Crop the extent of the target dataset to the extent of the cutline, setting the outside area to nodata",dest="cut_to_cutline",required=False,type=bool,default=True)
parser.add_argument("-cut_in_cutline","--cut_in_cutline",help="Crop the extent of the target dataset to the extent of the cutline, setting the inner area to nodata",dest="cut_in_cutline",required=False,type=bool,default=False)
args = parser.parse_args()

try:
    print("createJim")
    jim0=jl.createJim(args.input)
    rules=['centroid','min','max','mean','stdev']
    print("createVector")
    jlv=jl.createVector(args.extent)
    bb=jlv.getBoundingBox()
    if os.path.basename(args.extent)=='nuts_italy.sqlite':
        if args.cut_to_cutline:
            print("cut out")
            print("milano")
            jim_milano=jim0.cropOgr(jlv,{'ln':'milano','crop_to_cutline':True,'nodata':args.nodata,'align':True})
            bb=jim_milano.getBoundingBox()
            jim1=jim0.crop({'ulx':bb[0],'uly':bb[1],'lrx':bb[2],'lry':bb[3],'verbose':2})
            print(jim1.getBoundingBox())
            print(jim_milano.getBoundingBox())
            if jim1.getBoundingBox() != jim_milano.getBoundingBox():
                raise ValueError("Error: Bounding box milano not equal with crop and cropOgr")
            print("lodi")
            jim_lodi=jim0.cropOgr(jlv,{'ln':'lodi','crop_to_cutline':True,'nodata':args.nodata,'align':True})
        elif args.cut_in_cutline:
            print("cut within")
            # to keep region
            jim_milano=jim0.cropOgr(jlv,{'ln':'milano','crop_in_cutline':True,'nodata':args.nodata,'align':True})
            jim_lodi=jim0.cropOgr(jlv,{'ln':'lodi','crop_in_cutline':True,'nodata':args.nodata,'align':True})
        milanofn=os.path.join(os.path.dirname(args.output),'milano.tif')
        print("write")
        lodifn=os.path.join(os.path.dirname(args.output),'lodi.tif')
        jim_milano.write({'filename':milanofn})
        jim_milano.close()
        jim_lodi.write({'filename':lodifn})
        jim_lodi.close()
    else:
        jim0.cropOgr(jlv,{'align':True}).write({'filename':args.output}).close()
    jim0.close()
    jlv.close()
    print("Success: crop")
except:
    print("Failed: crop")



###############################################################################
# pytest_open.py: open raster dataset and calculate basic statistics
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
#
# This file is part of jiplib
###############################################################################

# History
# 2017/10/23 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log

# Test1: simple open and close
# Open an existing JP2 raster dataset
# get maximum pixel value in raster dataset
# close raster dataset

import argparse
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input file",dest="input",required=True,type=str)
args = parser.parse_args()

jim0=jl.createJim(args.input)
# Get basic statistics of the raster datasets
theStats=jim0.getStats({'function':['min','max','mean']})
if max(theStats['min'])!=0:
    print("Failed: getMin()")
elif min(theStats['max'])!=73:
    print("Failed: getMax()")
elif theStats['mean'][0]<16.9 or theStats['mean'][0]>17:
    print("Failed: getMean()")
else:
    print("Success: open raster dataset and calculate basic statistics")
jim0.close()



###############################################################################
# pytest_pytest_np.py: jim2np and np2jim
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2018/10/04 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import math
import numpy as np
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input raster dataset",dest="input",required=True,type=str)
parser.add_argument("-filter","--filter",help="filter function",dest="filter",required=False,type=str)
parser.add_argument("-band","--band",help="band to filter (leave empty to filter all bands)",dest="band",required=False,type=int,nargs='+')
parser.add_argument("-size","--size",help="size",dest="size",required=False,type=int,default=3)
parser.add_argument("-nodata","--nodata",help="no data value",dest="nodata",required=False,type=int,nargs='+')
parser.add_argument("-output","--output",help="Path of the classification output raster dataset",dest="output",required=False,type=str)
args = parser.parse_args()

# try:
if True:
    jimlist=jl.createJimList()
    if args.band:
        for band in args.band:
            jim0=jl.createJim(filename=args.input,band=band)
            print("mean of jim0:{}".format(jim0.getStats({'function':'mean'})))
            projection=jim0.getProjection()
            gt=jim0.getGeoTransform()
            np0=jl.jim2np(jim0)
            jim=jl.np2jim(np0)
            print("mean of jim:{}".format(jim.getStats({'function':'mean'})))
            print("number of cols in jim:{}".format(jim.nrOfCol()))
            print("number of rows in jim:{}".format(jim.nrOfRow()))
            # jim.setGeoTransform(gt)
            # jim.setProjection(projection)
            if args.filter:
                if args.nodata:
                    print("filter with nodata value {}".format(args.nodata))
                    jimlist.pushImage(jim.filter2d({'filter':args.filter,'dx':args.size,'dy':args.size,'nodata':args.nodata}))
                else:
                    print("filter")
                    jimlist.pushImage(jim.filter2d({'filter':args.filter,'dx':args.size,'dy':args.size}))
            else:
                print("jim.crop.nrOfCol():{}".format(jim.crop({}).nrOfCol()))
                jimlist.pushImage(jim.crop({}))
        print("stacking {} bands".format(jimlist.getSize()))
        jim=jimlist.stack({'verbose':2})
        print("number of cols in jim:{}".format(jim.nrOfCol()))
        print("number of rows in jim:{}".format(jim.nrOfRow()))
        print("mean of stacked jim:{}".format(jim.getStats({'function':'mean'})))
    else:
        jim0=jl.createJim(filename=args.input)
        projection=jim0.getProjection()
        gt=jim0.getGeoTransform()
        np0=jl.jim2np(jim0)
        jim=jl.np2jim(np0)
        # jim.setGeoTransform(gt)
        # jim.setProjection(projection)
        if args.filter:
            if args.nodata:
                print("filter with nodata value {}".format(args.nodata))
                jim=jim.filter2d({'filter':args.filter,'dx':args.size,'dy':args.size,'nodata':args.nodata})
            else:
                print("filter")
                jim=jim.filter2d({'filter':args.filter,'dx':args.size,'dy':args.size})
    print("number of cols in jim:{}".format(jim.nrOfCol()))
    print("number of rows in jim:{}".format(jim.nrOfRow()))
    print("mean of stacked jim:{}".format(jim.getStats({'function':'mean'})))
    print("number of output bands:{}".format(jim.nrOfBand()))
    print(jim.getStats({'function':'mean'}))
    if args.output:
        print("write output to {}".format(args.output))
        jim.write({'filename':args.output,'co':['COMPRESS=LZW','TILED=YES']})
    jim.close()
    print("Success: filter2d")
try:
    print("ok")
except:
    print("Failed: filter2d")



###############################################################################
# pytest_downsample.py: 100 m)
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/23 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input file",dest="input",required=True,type=str)
args = parser.parse_args()

# jim0=jl.createJim(filename=args.input,dx=1000,dy=1000)
jim0=jl.Jim({'filename':args.input,'dx':1000,'dy':1000})
if jim0.nrOfCol() == 256:
        print("Success: createJim with downsample")
else:
    print("Failed: createJim with downsample, ncol:",jim0.nrOfCol())




###############################################################################
# pytest_filter1d.py: filter1d
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/11/09 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import math
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input raster dataset",dest="input",required=True,type=str)
parser.add_argument("-filter","--filter",help="filter function",dest="filter",required=False,type=str,default='median')
parser.add_argument("-nodata","--nodata",help="no data value",dest="nodata",required=False,type=int,nargs='+')
parser.add_argument("-dz","--dz",help="kernel size",dest="dz",required=False,type=int,default=3)
parser.add_argument("-tapz","--tapz",help="custom filter taps",dest="tapz",required=False,type=float,nargs='+')
parser.add_argument("-threshold","--threshold",help="threshold for dwt cut",dest="threshold",required=False,type=float,default=10)
parser.add_argument("-output","--output",help="Path of the classification output raster dataset",dest="output",required=False,type=str)
parser.add_argument("-interp","--interp",help="Interpolation type",dest="interp",required=False,type=str,default="linear")
args = parser.parse_args()

try:
    print("create jim")
    jim=jl.createJim(args.input)
    print("created jim")
    print("nbands:",jim.nrOfBand())
    if args.tapz:
        jim_filtered=jim.filter1d({'tapz':args.tapz})
    else:
        if args.nodata:
            jim_filtered=jim.filter1d({'filter':args.filter,'dz':args.dz,'threshold':args.threshold,'nodata':args.nodata,'interp':args.interp})
        else:
            jim_filtered=jim.filter1d({'filter':args.filter,'dz':args.dz,'threshold':args.threshold,'otype':'Int16'}).pushNoDataValue(0).setThreshold({'min':0,'max':255}).convert({'otype':'Byte'})
    if args.output:
        jim_filtered.write({'filename':args.output})
    jim_filtered.close()
    jim.close()
    print("Success: filter1d")
except:
    print("Failed: filter1d")



###############################################################################
# pytest_createreference.py: Create a reference image
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/24 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log

import argparse
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-nrow","--nrow",help="Number of rows",dest="nrow",required=False,type=int,default=1024)
parser.add_argument("-ncol","--ncol",help="Number of cols",dest="ncol",required=False,type=int,default=1024)
parser.add_argument("-min","--min",help="min value for pixel values",dest="min",required=False,type=float,default=1000)
parser.add_argument("-max","--max",help="max value for pixel values",dest="max",required=False,type=float,default=2000)
args = parser.parse_args()

ULX=600000.0
ULY=4000020.0
LRX=709800.0
LRY=3890220.0
projection='epsg:32612'
dict={'ulx':ULX,'uly':ULY,'lrx':LRX,'lry':LRY,'a_srs':projection}
dict.update({'nrow':args.nrow,'ncol':args.ncol})
dict.update({'uniform':[args.min,args.max+1]})
dict.update({'otype':'GDT_UInt16'})
dict.update({'seed':10915})
try:
    jim0=jl.createJim(**dict)
    #create a reference
    jim1=jim0
    jim0.d_pointOpBlank(500)
    print(jim0)
    if not jim1.isEqual(jim0):
        print("Failed: isEqual")
        throw()
    theStats0=jim0.getStats({'function':['max']})
    theStats1=jim1.getStats({'function':['max']})
    jim0=None
    if theStats0['max']!=500:
        print("Failed: jim0 max")
        throw()
    else:
        print("theStats0: ",theStats0)
    print("debug5")
    if theStats1['max']!=500:
        print("Failed: jim1 max")
        throw()
    else:
        print("theStats1: ",theStats1)
    print("Success: Create a reference image")
except:
    print("Failed: Create a reference image")
jim1.close()



###############################################################################
# pytest_filter2d.py: filter2d
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/11/07 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import math
import jiplib as jl
import numpy as np

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input raster dataset",dest="input",required=True,type=str)
parser.add_argument("-filter","--filter",help="filter function",dest="filter",required=False,type=str,default='median')
parser.add_argument("-band","--band",help="band to filter (leave empty to filter all bands)",dest="band",required=False,type=int,nargs='+')
parser.add_argument("-size","--size",help="filter size",dest="size",required=False,type=int,default=3)
parser.add_argument("-cut","--cut",help="cut percentage of wavelet coefficients",dest="cut",required=False,type=int,default=0)
parser.add_argument("-nodata","--nodata",help="no data value",dest="nodata",required=False,type=int,nargs='+')
parser.add_argument("-output","--output",help="Path of the classification output raster dataset",dest="output",required=False,type=str)
args = parser.parse_args()

try:
    jim=jl.createJim(args.input)
    if args.band:
        jim=jim.cropBand({'band':args.band})
    if 'tap' in args.filter:
        taps=np.array([[-1.0,0.0,1.0],[-2.0,0.0,2.0],[-1.0,0.0,1.0]])
        fdict={}
        fdict.update({'dy': taps.shape[0]})
        fdict.update({'dx': taps.shape[1]})
        fdict.update({'tap':taps.flatten().tolist()})
        fdict.update({'verbose':2})
        jim=jim.convert({'otype':'GDT_Float32'})
        print(jim.getStats({}))
        jim_filtered=jim.filter2d(fdict)
        print(jim_filtered.getStats({}))
        # jimdx.ngbops.filter2d(tapsdx,nodata=jim_object.properties.getNoDataVals(),abs=True,norm=True)
    elif 'dwt' in args.filter:
        if args.cut > 0:
            jim_filtered=jim.filter2d({'filter':'dwt_cut','otype':'Int16','threshold':args.cut}).pushNoDataValue(0).setThreshold({'min':0,'max':255}).convert({'otype':'Byte'})
        else:
            jim_filtered=jim.filter2d({'filter':'dwt','otype':'Int16'})
            jim_filtered=jim_filtered.filter2d({'filter':'dwti'}).pushNoDataValue(0).setThreshold(0,255).convert({'otype':'Byte'})
    else:
        if args.nodata:
            jim_filtered=jim.filter2d({'filter':args.filter,'dx':args.size,'dy':args.size,'nodata':args.nodata})
        else:
            jim_filtered=jim.filter2d({'filter':args.filter,'dx':args.size,'dy':args.size})
    if args.output:
        jim_filtered.write({'filename':args.output,'co':['COMPRESS=LZW','TILED=YES']})
    jim_filtered.close()
    jim.close()
    print("Success: filter2d")
except:
    print("Failed: filter2d")



###############################################################################
# pytest_setmask.py: setMask with vector
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/24 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the raster dataset",dest="input",required=True,type=str)
parser.add_argument("-vectormask","--vectormask",help="Path of the vector mask dataset",dest="vm",required=False,type=str)
args = parser.parse_args()

jim0=jl.createJim(args.input)
if args.vm:
    try:
        print("create vector")
        v0=jl.createVector()
        print("open vector", args.vm)
        v0.open({'filename':args.vm,'noread':True})
        print("setMask")
        jim1=jim0.setMask(v0,{'nodata':255,'eo':'ALL_TOUCHED'})
        if jim1.getNvalid()!=250568:
            print(jim1.getNvalid())
            print("Failed: nvalid",jim1.getNvalid())
            throw()
        if jim1.getNinvalid()!=11576:
            print(jim1.getNinvalid())
            print("Failed: ninvalid",jim1.getNinvalid())
            throw()
        print("Success: setMask with vector")
    except:
        print("Failed: setMask with vector")
else:
    try:
        jim1=jim0.getMask({'min':1,'max':20,'nodata':0,'data':1}).pushNoDataValue(1)
        jl1=jl.JimList([jim1])
        jim2=jim0.setMask(jl1,{'msknodata':0})
        if jim2.getNvalid()!=68335:
            print("Failed: nvalid",jim1.getNvalid())
            throw()
        if jim2.getNinvalid()!=193809:
            print("Failed: ninvalid",jim1.getNinvalid())
            throw()
        print("Success: setMask")
    except:
        print("Failed: setMask")
jim0.close()
jim1.close()



###############################################################################
# pytest_extractogr.py: extractogr
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
#
# This file is part of jiplib
###############################################################################

# History
# 2017/10/24 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import math
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the raster dataset",dest="input",required=True,type=str)
parser.add_argument("-vector","--vector",help="Path of the vector dataset",dest="vector",required=False,type=str)
parser.add_argument("-output","--output",help="Path of the output vector dataset",dest="output",required=False,type=str)
parser.add_argument("-noread","--noread",help="Postpone reading raster dataset",dest="noread",required=False,type=bool,default=False)
parser.add_argument("-random","--random",help="Number of random pixels to select",dest="random",required=False,type=int,default=100)
args = parser.parse_args()

try:
    jim0=jl.createJim(args.input)
    rules=['centroid','min','max','mean','stdev']
    if not args.vector:
        v01=jim0.extractSample({'random':20,'buffer':3,'rule':rules,'output':'mem01','oformat':'Memory'})
        v01.close()
        npoint=100
        gridsize=int(jim0.nrOfCol()*jim0.getDeltaX()/math.sqrt(npoint))
        v02=jim0.extractSample({'grid':gridsize,'buffer':3,'rule':rules,'output':'mem02','oformat':'Memory'})
        v02.close()
        if args.output:
            v1=jim0.extractSample({'grid':gridsize,'rule':'point','output':args.output,'oformat':'SQLite'})
            v1.write()
            v1.close()
    else:
        sample=jl.createVector(args.vector);
        if os.path.basename(args.vector)=='nuts_italy.sqlite':
            if args.noread:
                print("createVector")
                v0=jl.createVector(args.vector)
                print("crop Milano")
                jim_milano=jim0.cropOgr(v0,{'ln':'milano','align':True})
                milanofn=os.path.join(os.path.dirname(args.output),'milano.tif')
                print("write Milano")
                jim_milano.write({'filename':milanofn})
                print("close Milano")
                jim_milano.close()
                print("crop Lodi")
                jim_lodi=jim0.cropOgr(v0,{'ln':'lodi','align':True})
                lodifn=os.path.join(os.path.dirname(args.output),'lodi.tif')
                print("write Lodi")
                jim_lodi.write({'filename':lodifn})
                print("close Lodi")
                jim_lodi.close()
                jimlist=jl.JimList([jl.createJim(milanofn,noread=True),jl.createJim(lodifn,noread=True)])
                print("extractOgr")
                v2=jimlist.extractOgr(sample,{'rule':rules[3],'output':args.output,'oformat':'SQLite','co':'OVERWRITE=YES','all_covered':True})
                print("write v2")
                v2.write()
                print("close v2")
                v2.close()
                print("close jim_milano")
                jim_milano.close()
                print("close jim_lodi")
                jim_lodi.close()
                v0.close()
            else:
                v2=jim0.extractOgr(sample,{'rule':rules[2],'output':args.output,'oformat':'SQLite','co':'OVERWRITE=YES'})
                vposbuffer=jim0.extractOgr(sample,{'rule':rules[2],'output':'/vsimem/posbuffer.sqlite','oformat':'SQLite','co':'OVERWRITE=YES', 'buffer':1000})
                vnegbuffer=jim0.extractOgr(sample,{'rule':rules[2],'output':'/vsimem/negbuffer','oformat':'SQLite','co':'OVERWRITE=YES', 'buffer':-1000})
                v2.write()
                v2.close()
                vposbuffer.close()
                vnegbuffer.close()
        else:
            v=jl.createVector()
            for band in range(0,11):
                print(band)
                print("create jimlist")
                jl0=jl.JimList([jim0.cropBand({'band':band})])
                bandname='B'+str(band)
                print("bandname: ",bandname)
                if not band:
                    print("first time")
                    print("extractOgr")
                    v=jl0.extractOgr(sample,{'rule':'mean','output':args.output,'oformat':'SQLite','co':['OVERWRITE=YES'],'bandname':bandname,'fid':'fid'})
                    v.write()
                    v.close()
                else:
                    v1=jl.createVector(args.output)
                    v2=jl0.extractOgr(sample,{'rule':'mean','output':'/vsimem/v2.sqlite','oformat':'SQLite','co':['OVERWRITE=YES'],'bandname':bandname,'fid':'fid'})
                    v=v1.join(v2,{'output':args.output,'oformat':'SQLite','co':['OVERWRITE=YES'],'key':['fid']});
                    v1.close()
                    v2.close()
                    v.write()
                    v.close()
            jl0.close()
        sample.close()
    jim0.close()
    print("Success: extractogr")
except:
    print("Failed: extractogr")



###############################################################################
# pytest_upsample.py: 5 m)
# Author(s): Pieter.Kempeneers@ec.europa.eu
# Copyright (c) 2016-2019 European Union (Joint Research Centre)
# License EUPLv1.2
# 
# This file is part of jiplib
###############################################################################

# History
# 2017/10/23 - Created by Pieter Kempeneers (pieter.kempeneers@ec.europa.eu)
# Change log


import argparse
import os
import jiplib as jl

parser=argparse.ArgumentParser()
parser.add_argument("-input","--input",help="Path of the input file",dest="input",required=True,type=str)
args = parser.parse_args()

jim0=jl.createJim(filename=args.input,noread=True)
ULX=jim0.getUlx()
ULY=jim0.getUly()
LRX=ULX+1000
LRY=ULY-1000
jim0.close()
jim0=jl.createJim(filename=args.input,ulx=ULX,uly=ULY,lrx=LRX,lry=LRY,dx=5,dy=5)
if jim0.nrOfCol() != 200:
    print("Failed: createJim with upsample")
elif jim0.nrOfRow() != 200:
    print("Failed: createJim with upsample")
else:
    print("Success: createJim with upsample")
jim0.close()

